#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/wdt.h>

#define F_CPU	8000000UL
#define PRESCALER1	8UL
#define F_TIMER1	(100000UL)


volatile unsigned int pulsein = 0;
volatile unsigned char pulse = 0;
volatile unsigned char sound = 0;
volatile unsigned int soundpresc = 0;
volatile unsigned char watch = 0;
volatile unsigned int fail = 0;

SIGNAL (SIG_OUTPUT_COMPARE1A)	//Alle 10us
{

	if(PINC & (1<<PC0)){	//Pulsdauer messen
		pulsein++;
	}else if(pulsein!=0){
		if(pulsein>255)
			pulse=0;
		else
			pulse=pulsein;
		pulsein=0;
	}
	
	if(soundpresc++ == 140){		//Tonausgabe
		soundpresc=0;
		if(sound)
			PORTD ^= (1<<PD7);		//"Ton" ausgeben
		else
			PORTD &= ~(1<<PD7);	//Sicherstellen, dass der FET sperrt
	}
	
	watch = 1;				//Signal fuer Hauptschlaeife, den Watchdog zu fuettern
	fail++;					//Zeitramen fuer Plausibilitaetstest erhoehen
}


int main(void)
{
	unsigned int mid = 0;
	unsigned char i;
	unsigned char chan1wait = 0;
	
	DDRC = 0;
	PORTC = 0;						//Alles Eingaenge
	DDRD |= (1<<PD7)|(1<<PD6);		//FET Ausgaenge
	PORTD = 0;						//Alles aus
	
	OCR1A = (F_CPU/PRESCALER1/F_TIMER1)-1L;	//Freequenz einstellen
	TCCR1B |= (1 << CS11) /*| (1 << CS10)*/ | (1 << WGM12);	//Timer1 CTC /8
	TIMSK |= (1 << OCIE1A);
	
	wdt_enable( WDTO_15MS);		//Watchdog einschalten
	sei();
	
	for(i=0;i<60;i++){			//Mittlere Periodenlaenge ueber 60 Pulse messen
		while(!pulse);
		mid += pulse;
	}
	mid /= 60;
	
	
	while(1){
		if(pulse!=0){				//Neuer Eingangsmpuls?
			if(fail > 4000){	//In einem gueltigen Zeitramen?(40ms)
				fail=0;
				continue;
			}
			fail = 0;			//Zeitramen Zuruecksetzen
			
			if(pulse>mid+20 && !chan1wait){	//Knueppel nach unten und vorher in Mittelstellung?
				PORTD ^= (1<<PD6);			//Licht an
				chan1wait = 1;				//Auf Mittelstellung warten
			}
			if(pulse>mid-10 && pulse<mid+10){		//Knueppel in MIttelstellung?
				chan1wait = 0;				//Flag setzen
				sound = 0;					//Ton aus
			}
			if(pulse<mid-20)					//Knueppel oben?
				sound = 1;					//Ton an
			pulse=0;
		}
		
		if(watch){							//Watchdog fuettern, falls der Interrupt und die Mainschleife noch laufen
			watch = 0;
			wdt_reset();
		}
		if(fail > 0XFF00){					//Falls kein Signal mehr alles ausschalen
			sound = 0;
			PORTD &= ~(1<<PD6);
		}
			
	}
}
